import { world, system, BlockPermutation, ItemStack, GameMode } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:on_player_place', {
        beforeOnPlayerPlace: e => {
            const { block } = e;
            const { x, y, z } = block.location;
            if (block.typeId.includes("water") || block.typeId.includes("lava")) {
                e.cancel = true;
            }
            else {
                return;
            }
        }
    });
});




world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
    blockComponentRegistry.registerCustomComponent("ff:switch", {
        onPlayerInteract: e => {
            const { player, block } = e;

            if (player.isSneaking) {
                e.cancel = true;
                return;
            }

            const inventory = player.getComponent("minecraft:inventory").container;
            const slot = player.selectedSlotIndex;
            const item = (typeof slot === "number" && slot >= 0 && slot < inventory.size)
                ? inventory.getItem(slot)
                : null;

            if (item && item.typeId === "minecraft:breeze_rod") {
                player.playSound("random.pop2");
                return;
            }

            player.playSound("random.click");
            const enable = block.permutation.withState("ff:switch_type", true);
            const disable = block.permutation.withState("ff:switch_type", false);

            if (block.permutation.getState("ff:switch_type") === false) {
                block.setPermutation(enable);
                return;
            }
            if (block.permutation.getState("ff:switch_type") === true) {
                block.setPermutation(disable);
                return;
            }
        }
    });
});


  world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
    blockComponentRegistry.registerCustomComponent("ff:single_interactive", {
      onPlayerInteract: e => {
        const { player, block } = e;
        player.playSound("random.click");
      }
    });
  });




  

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:toilet_function', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const toilet_open = block.permutation.withState("ff:bathroom_vars", 1);
            const toilet_function = block.permutation.withState("ff:bathroom_vars", 2);
            const toilet_close = block.permutation.withState("ff:bathroom_vars", 0);
            if (player.isSneaking && block.permutation.getState("ff:bathroom_vars") === 0) {
                block.setPermutation(toilet_open);
                return;
            }
            if (player.isSneaking && block.permutation.getState("ff:bathroom_vars") === 1) {
                block.setPermutation(toilet_function);
                player.playSound("ff:toilet");
                return;
            }
            if (player.isSneaking && block.permutation.getState("ff:bathroom_vars") === 2) {
                block.setPermutation(toilet_close);
                return;
            }
        }
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:interactive_block', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const updates = e.block.permutation.getState("ff:lamp_state");
            const update = block.permutation.withState("ff:lamp_state", updates + 1);
            block.setPermutation(update);
            player.playSound("random.click");
        }
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    const registry = ffh.blockComponentRegistry;

    registry.registerCustomComponent("ff:summoning_light", {
        onPlace: e => {
            const { block } = e;
            const { x, y, z } = block.location;

            const name = block.typeId.split(":")[1];
            const isFan = name.includes("fan");
            const entityType = isFan
                ? "ff:ff_ceiling_fan"
                : "ff:ff_ceiling_light";

            const cx = x + 0.5;
            const cy = isFan ? y - 0 : y;
            const cz = z + 0.5;

            if (block.permutation.getState("ff:lamp_state")) {
                block.dimension.runCommand(
                    `summon ${entityType} ${cx} ${cy} ${cz}`
                );
            }
        },
        onPlayerDestroy: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            
            const name = block.typeId.split(":")[1];
            const isFan = name.includes("fan");
            const entityType = isFan
                ? "ff:ff_ceiling_fan"
                : "ff:ff_ceiling_light";

            const cx = x + 0.5;
            const cy = isFan ? y - 0 : y;
            const cz = z + 0.5;

            block.dimension.runCommand(
                `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=${entityType},r=1] destroy`
            );
        },
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            player.playSound("random.click");

            const currentState = block.permutation.getState("ff:lamp_state");
            const newState = !currentState;
            block.setPermutation(
                block.permutation.withState("ff:lamp_state", newState)
            );

            const name = block.typeId.split(":")[1];
            const isFan = name.includes("fan");
            const entityType = isFan
                ? "ff:ff_ceiling_fan"
                : "ff:ff_ceiling_light";

            const cx = x + 0.5;
            const cy = isFan ? y - 0 : y;
            const cz = z + 0.5;

            if (newState) {
                block.dimension.runCommand(
                    `summon ${entityType} ${cx} ${cy} ${cz}`
                );
            } else {
                block.dimension.runCommand(
                    `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=${entityType},r=1] destroy`
                );
            }
        },
        onTick: e => {
            const { block } = e;
            const { x, y, z } = block.location;

            const name = block.typeId.split(":")[1];
            const isFan = name.includes("fan");
            const entityType = isFan
                ? "ff:ff_ceiling_fan"
                : "ff:ff_ceiling_light";

            const cx = x + 0.5;
            const cy = isFan ? y - 0 : y;
            const cz = z + 0.5;

            if (!isFan && block.permutation.getState("ff:lamp_state")) {
                block.dimension.runCommand(
                    `particle ff:ff_light_ray ${x} ${y + 0.8} ${z}`
                );
            }

            if (isFan && block.permutation.getState("ff:lamp_state")) {
                block.dimension.runCommand(
                    `particle ff:ff_light_ray ${x} ${y + 0} ${z}`
                );
            }
        }
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:paper_fill', {
      onPlayerInteract: e => {
        const { player, block } = e;
        const loc = block.location;
        const dim = block.dimension;
        const equip = player.getComponent('equippable');
        const main = equip.getEquipment('Mainhand');
        const state = block.permutation.getState('ff:paper');
  
        const spawn = (item) => dim.spawnItem(item, { x: loc.x + 0.5, y: loc.y + 1, z: loc.z + 0.5 });
        const paper = new ItemStack('minecraft:paper');
        const tpItem = new ItemStack('ff:toilet_paper_item');
  
        if (main?.typeId === 'ff:toilet_paper_item' && state === 0) {
          block.setPermutation(block.permutation.withState('ff:paper', 9));
          if (main.amount > 1) {
            main.amount -= 1;
            equip.setEquipment('Mainhand', main);
          } else {
            equip.setEquipment('Mainhand', undefined);
          }
          return;
        }
  
        if (state > 0) {
          if (player.isSneaking) {
            block.setPermutation(block.permutation.withState('ff:paper', 0));
            spawn(tpItem);
          } else {
            block.setPermutation(block.permutation.withState('ff:paper', state - 1));
            if (state > 1) spawn(paper);
          }
        }
      }
    });
  });
  


world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:add_new_stacked_book', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const stacked_book = e.block.permutation.getState("book:state");
            const book_s = block.permutation.withState("book:state", stacked_book + 1);
            const book_remove = block.permutation.withState("book:vertical_stacked", + 1);
            const equipment = player.getComponent('equippable');
            const selectedItem = equipment.getEquipment('Mainhand');
            if (selectedItem && (selectedItem.typeId === 'ff:stacked_books') && block.permutation.getState("book:state") < 3) {
                player.playSound("use.candle");
                block.setPermutation(book_s);
                if (selectedItem.amount > 1) {
                    selectedItem.amount -= 1;
                    equipment.setEquipment('Mainhand', selectedItem);
                } else {
                    equipment.setEquipment('Mainhand', undefined);
                }
                return;
            }
            if (player.isSneaking && block.permutation.getState("book:state") === 3) {
                player.playSound("fall.wood");
                block.setPermutation(book_remove);
                return;

            }
        }
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:sink_open_close', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const sink_open = block.permutation.withState("ff:sink_vars", 1);
            const sink_close = block.permutation.withState("ff:sink_vars", 0);
            if (block.permutation.getState("ff:sink_vars") === 0) {
                block.setPermutation(sink_open);
                player.playSound("mob.axolotl.splash");
                return;
            }
            if (block.permutation.getState("ff:sink_vars") === 1) {
                block.setPermutation(sink_close);
                return;
            }
        },
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:trash_items', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const equipment = player.getComponent('equippable');
            const selectedItem = equipment.getEquipment('Mainhand');

            if (!player.isSneaking && selectedItem) {
                player.playSound("random.pop2");
                if (selectedItem.amount > 1) {
                    selectedItem.amount -= 1;
                    equipment.setEquipment('Mainhand', selectedItem);
                } else {
                    equipment.setEquipment('Mainhand', undefined);
                }
                return;
            }
        }
    });
});


// New dynamic weather system - thanks to Reda karimi for the help!
world.beforeEvents.weatherChange.subscribe(({ newWeather }) => {
    system.run(() => {
        world.setDynamicProperty("ff:dynamic_weather_system", newWeather);
    });
});

world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry: registry }) => {
    registry.registerCustomComponent("ff:tv_change_channel", {
        onPlayerInteract({ block, player }) {
            const current = block.permutation.getState("ff:channels");
            const nextPerm = block.permutation.withState("ff:channels", current === 0 ? 1 : 0);
            player.playSound("random.click");
            block.setPermutation(nextPerm);
        }
    });

    registry.registerCustomComponent("ff:tv_weather_channel", {
        onTick({ block }) {
            const current = block.permutation.getState("ff:channels");
            const perms = {
                clear:   block.permutation.withState("ff:channels", 1),
                rain:    block.permutation.withState("ff:channels", 2),
                thunder: block.permutation.withState("ff:channels", 3),
                snow:    block.permutation.withState("ff:channels", 4),
            };
            const weather = world.getDynamicProperty("ff:dynamic_weather_system");

            if (weather === "Rain"    && (current === 1 || current === 3)) block.setPermutation(perms.rain);
            else if (weather === "Thunder" && (current === 1 || current === 2)) block.setPermutation(perms.thunder);
            else if (weather === "Snow"    &&  current === 1)                      block.setPermutation(perms.snow);
            else if (weather === "Clear"   && (current >= 2 && current <= 4))      block.setPermutation(perms.clear);
        }
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    const woodTypes = [
        'jungle','birch','crimson','warped',
        'cherry','mangrove','oak','dark_oak',
        'acacia','pale','spruce','cinder','spicewood'
    ];
    const directions = ['north','south','east','west'];

    woodTypes.forEach(type => {
        ffh.blockComponentRegistry.registerCustomComponent(
            `ff:${type}_add_couch`,
            {
                onPlayerInteract: e => {
                    const { player, block } = e;
                    const { x, y, z } = block.location;
                    const equip = player.getComponent('equippable');
                    const item = equip.getEquipment('Mainhand');

                    if (player.isSneaking || !item || item.typeId !== 'ff:white_cushion') return;

                    player.playSound('hit.cloth');
                    block.dimension.spawnParticle('ff:cushion_effect', {
                        x: x + 0.5,
                        y: y + 0.5,
                        z: z + 0.5
                    });

                    if (player.getGameMode() !== GameMode.creative) {
                        if (item.amount > 1) {
                            item.amount -= 1;
                            equip.setEquipment('Mainhand', item);
                        } else {
                            equip.setEquipment('Mainhand', undefined);
                        }
                    }

                    directions.forEach(dir => {
                        block.dimension.runCommand(
                            `fill ${x} ${y} ${z} ${x} ${y} ${z}` +
                            ` ff:wooden_${type}_chair_with_wool` +
                            `["minecraft:cardinal_direction"="${dir}"]` +
                            ` replace ff:wooden_${type}_chair` +
                            `["minecraft:cardinal_direction"="${dir}"]`
                        );
                    });
                }
            }
        );
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    const woodTypes  = [
        'jungle','birch','crimson','warped',
        'cherry','mangrove','oak','dark_oak',
        'acacia','pale','spruce','cinder','spicewood'
    ];
    const directions = ['north','south','east','west','up'];

    woodTypes.forEach(type => {
        ffh.blockComponentRegistry.registerCustomComponent(
            `ff:stool_${type}_add_couch`,
            {
                onPlayerInteract(e) {
                    const { player, block } = e;
                    if (player.isSneaking) return;

                    const equip = player.getComponent('equippable');
                    const item  = equip.getEquipment('Mainhand');
                    if (!item || item.typeId !== 'ff:white_cushion') return;

                    player.playSound('hit.cloth');
                    block.dimension.spawnParticle('ff:cushion_effect', {
                        x: block.location.x + 0.5,
                        y: block.location.y + 0.5,
                        z: block.location.z + 0.5
                    });

                    // Only consume cushion in Survival
                    if (player.getGameMode() !== GameMode.creative) {
                        if (item.amount > 1) {
                            item.amount--;
                            equip.setEquipment('Mainhand', item);
                        } else {
                            equip.setEquipment('Mainhand', undefined);
                        }
                    }

                    const rotation  = block.permutation.getState('ff:block_rotation');
                    const placement = block.permutation.getState('minecraft:block_face');
                    const { x, y, z } = block.location;

                    directions.forEach(dir => {
                        block.dimension.runCommand(
                            `fill ${x} ${y} ${z} ${x} ${y} ${z}` +
                            ` ff:wooden_${type}_stool_with_wool[` +
                                `"ff:block_rotation"=${rotation},` +
                                `"minecraft:block_face"="${dir}"` +
                            `] replace ff:wooden_${type}_stool[` +
                                `"ff:block_rotation"=${rotation},` +
                                `"minecraft:block_face"="${dir}"` +
                            `]`
                        );
                    });
                }
            }
        );
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    const woodTypes = [
      'jungle','birch','crimson','warped',
      'cherry','mangrove','oak','dark_oak',
      'acacia','pale','spruce','cinder','spicewood'
    ];
  
    function registerRemoveCouch(componentId, getReplaceCommand) {
      ffh.blockComponentRegistry.registerCustomComponent(componentId, {
        onPlayerInteract(e) {
          const player = e.player;
          const block  = e.block;
          if (!player.isSneaking) return;
  
          if (player.gameMode !== GameMode.creative) {
            block.dimension.spawnItem(
              new ItemStack("ff:white_cushion", 1),
              { x: block.location.x + 0.5, y: block.location.y + 0.5, z: block.location.z + 0.5 }
            );
          }
  
          const { x, y, z } = block.location;
          const rotation = block.permutation.getState("ff:block_rotation");
          const face = block.permutation.getState("minecraft:block_face");
          
          const newBlockType = block.typeId.replace("_with_wool", "");
          
          const baseCommand = `fill ${x} ${y} ${z} ${x} ${y} ${z} ${newBlockType} replace ${block.typeId}`;
          block.dimension.runCommand(baseCommand);
          
          const newBlock = block.dimension.getBlock({ x, y, z });
          if (!newBlock) return;
          
          try {
            if (rotation !== undefined) {
              newBlock.setPermutation(newBlock.permutation.withState("ff:block_rotation", rotation));
            }
            if (face !== undefined) {
              newBlock.setPermutation(newBlock.permutation.withState("minecraft:block_face", face));
            }
          } catch (error) {
            console.warn(`Error al aplicar estados: ${error}`);
          }
        },
  
        onPlayerDestroy(e) {
          const player = e.player;
          if (!player || player.getGameMode() === GameMode.creative) return;
  
          const destroyedPerm = e.destroyedBlockPermutation;
          if (!destroyedPerm) return;
  
          const oldId = destroyedPerm.type.id;
          const baseId = oldId.endsWith("_with_wool")
            ? oldId.replace("_with_wool", "")
            : oldId;
  
          const loc = {
            x: e.block.location.x + 0.5,
            y: e.block.location.y + 0.5,
            z: e.block.location.z + 0.5
          };
  
          e.dimension.spawnItem(new ItemStack(baseId, 1), loc);
          e.dimension.spawnItem(new ItemStack("ff:white_cushion", 1), loc);
        }
      });
    }
  
  
    // Stools
    for (const type of woodTypes) {
      registerRemoveCouch(
        `ff:stool_${type}_remove_couch`,
        block => {
          const rotation = block.permutation.getState("ff:block_rotation");
          const color = block.permutation.getState("ef:colors");
          const face     = block.permutation.getState("minecraft:block_face");
          const { x, y, z } = block.location;
          return `fill ${x} ${y} ${z} ${x} ${y} ${z}` +
                 ` ff:wooden_${type}_stool["ff:block_rotation"=${rotation},"minecraft:block_face"="${face}"]` +
                 ` replace ff:wooden_${type}_stool_with_wool["ff:block_rotation"=${rotation},"minecraft:block_face"="${face}"]`;
        }
      );
    }
  });


world.beforeEvents.worldInitialize.subscribe((ffh) => {
    const playerJumpCounts = new Map();
    
    ffh.blockComponentRegistry.registerCustomComponent(
      "ff:fan_elevador",
      {
        onStepOn: (event) => {
          const { block, entity } = event;
          
          if (!entity.isValid() || entity.typeId !== 'minecraft:player') {
            entity.applyImpulse({
              x: 0,
              y: 0.5,
              z: 0
            });
            return;
          }
          
          const currentCount = playerJumpCounts.get(entity.id) || 0;
          
          let impulseY = 0.5;
          if (currentCount > 0) {
            impulseY = Math.min(0.5 + (currentCount * 0.2), 3);
          }
          
          entity.applyImpulse({
            x: 0,
            y: impulseY,
            z: 0
          });
          
          playerJumpCounts.set(entity.id, currentCount + 1);
          
          system.runTimeout(() => {
            const currentCount = playerJumpCounts.get(entity.id);
            if (currentCount) {
              playerJumpCounts.set(entity.id, 0);
            }
          }, 50);
        },
      }
    );
  });

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:add_breads', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const equipment = player.getComponent('equippable');
            const selectedItem = equipment.getEquipment('Mainhand');
            const add_bread = block.permutation.withState("ff:toaster_with_breads", 1);
            const add_bread_2 = block.permutation.withState("ff:toaster_with_breads", 2);
            const remove_bread = block.permutation.withState("ff:toaster_with_breads", 0);
            const breadGive = new ItemStack("ff:bread_slice");
            const breadGive2 = new ItemStack("ff:bread_slice");
            if (!player.isSneaking && selectedItem && (selectedItem.typeId === 'ff:bread_slice') && block.permutation.getState("ff:toaster_with_breads") === 0) {
                block.setPermutation(add_bread);
                player.playSound("random.pop");
                if (selectedItem.amount > 1) {
                    selectedItem.amount -= 1;
                    equipment.setEquipment('Mainhand', selectedItem);
                } else {
                    equipment.setEquipment('Mainhand', undefined);
                }
                return;
            }
            if (!player.isSneaking && selectedItem && (selectedItem.typeId === 'ff:bread_slice') && block.permutation.getState("ff:toaster_with_breads") === 1) {
                block.setPermutation(add_bread_2);
                player.playSound("random.pop");
                if (selectedItem.amount > 1) {
                    selectedItem.amount -= 1;
                    equipment.setEquipment('Mainhand', selectedItem);
                } else {
                    equipment.setEquipment('Mainhand', undefined);
                }
                return;
            }
            if (!player.isSneaking && block.permutation.getState("ff:toaster_with_breads") === 2) {
                block.setPermutation(remove_bread);
                player.playSound("random.pop2");
                block.dimension.spawnItem(breadGive, { x: block.location.x + 0.5, y: block.location.y + 1, z: block.location.z + 0.5 });
                block.dimension.spawnItem(breadGive2, { x: block.location.x + 0.5, y: block.location.y + 1, z: block.location.z + 0.5 });
            }
        },
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:outlet_function', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const equipment = player.getComponent('equippable');
            const selectedItem = equipment.getEquipment('Mainhand');
            const add_folk = block.permutation.withState("ff:outlet_function", 1);
            const remove_folk = block.permutation.withState("ff:outlet_function", 0);
            const givefolk = new ItemStack("ff:folk");
            if (!player.isSneaking && selectedItem && (selectedItem.typeId === 'ff:folk') && block.permutation.getState("ff:outlet_function") === 0) {
                block.setPermutation(add_folk);
                player.playSound("random.pop2");
                if (selectedItem.amount > 1) {
                    selectedItem.amount -= 1;
                    equipment.setEquipment('Mainhand', selectedItem);
                } else {
                    equipment.setEquipment('Mainhand', undefined);
                }
                return;
            }
            if (!player.isSneaking && block.permutation.getState("ff:outlet_function") === 1) {
                block.dimension.spawnItem(givefolk, { x: block.location.x + 0.5, y: block.location.y + 1, z: block.location.z + 0.5 });
                block.setPermutation(remove_folk);
                player.playSound("random.pop");

                return;
            }
            if (!player.isSneaking && block.permutation.getState("ff:outlet_function") === 2) {
                block.dimension.runCommand(`title @p actionbar §6 It's too late...`);
                return;
            }
        },
        onTick: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const folk_explode = block.permutation.withState("ff:outlet_function", 2);
            const folk_explode2 = block.permutation.withState("ff:outlet_function", 3);
            if (block.permutation.getState("ff:outlet_function") === 1) {
                block.dimension.spawnParticle("ff:smoke_folk", { x: block.location.x + 0.5, y: block.location.y + 0.5, z: block.location.z + 0.5 });
                block.setPermutation(folk_explode);
                return;
            }
            if (block.permutation.getState("ff:outlet_function") === 2) {
                block.dimension.spawnParticle("ff:elec_folk", { x: block.location.x + 0.5, y: block.location.y + 0.5, z: block.location.z + 0.5 });
                block.dimension.spawnParticle("ff:smoke_folk", { x: block.location.x + 0.5, y: block.location.y + 0.5, z: block.location.z + 0.5 });
                block.setPermutation(folk_explode2);
                return;
            }
            if (block.permutation.getState("ff:outlet_function") === 3) {
                block.dimension.runCommand(`summon ender_crystal ${x} ${y} ${z} 0 0 minecraft:crystal_explode`);
                block.dimension.runCommand(`setblock ${x} ${y} ${z} air`)
                block.dimension.spawnParticle("ff:elec_folk", { x: block.location.x + 0.5, y: block.location.y + 0.5, z: block.location.z + 0.5 });
                block.dimension.spawnParticle("ff:smoke_folk", { x: block.location.x + 0.5, y: block.location.y + 0.5, z: block.location.z + 0.5 });
                return;
            }
        }
    });
});


world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:give_breads', {
        onPlayerInteract: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const execute_toaster1 = block.permutation.withState("ff:toaster_with_breads", 3);
            const execute_toaster2 = block.permutation.withState("ff:toaster_with_breads", 4);
            if (player.isSneaking && block.permutation.getState("ff:toaster_with_breads") === 1) {
                block.setPermutation(execute_toaster1);
                player.playSound("hit.netherite");
            }
            if (player.isSneaking && block.permutation.getState("ff:toaster_with_breads") === 2) {
                block.setPermutation(execute_toaster2);
                player.playSound("hit.netherite");
            }
        },
        onTick: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            const breadtoastedGive = new ItemStack("ff:bread_slice_toasted");
            const breadtoastedGive2 = new ItemStack("ff:bread_slice_toasted");
            const finish_execute_toaster1 = block.permutation.withState("ff:toaster_with_breads", 0);
            const finish_execute_toaster2 = block.permutation.withState("ff:toaster_with_breads", 0);
            if (block.permutation.getState("ff:toaster_with_breads") === 3) {
                block.setPermutation(finish_execute_toaster1);

                block.dimension.runCommand(`playsound ff:toast_finish @p`);
                block.dimension.spawnItem(breadtoastedGive, { x: block.location.x + 0.5, y: block.location.y + 1, z: block.location.z + 0.5 });
            }
            if (block.permutation.getState("ff:toaster_with_breads") === 4) {
                block.setPermutation(finish_execute_toaster2);
                block.dimension.runCommand(`playsound ff:toast_finish @p`);
                block.dimension.spawnItem(breadtoastedGive, { x: block.location.x + 0.5, y: block.location.y + 1, z: block.location.z + 0.5 });
                block.dimension.spawnItem(breadtoastedGive2, { x: block.location.x + 0.5, y: block.location.y + 1, z: block.location.z + 0.5 });
            }
            if (Math.random() < 0.09 && block.permutation.getState("ff:toaster_with_breads") === 3) {
                block.dimension.runCommand(
                  `particle ff:pan_smoke ${x} ${y + 0.1} ${z}`
                );
            }
            if (Math.random() < 0.09 && block.permutation.getState("ff:toaster_with_breads") === 4) {
                block.dimension.runCommand(
                  `particle ff:pan_smoke ${x} ${y + 0.1} ${z}`
                );
            }
        },
    });
});

world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:smoke_toaster', {
        onTick: e => {
            const { player, block } = e;
            const { x, y, z } = block.location;
            if (block.permutation.getState("ff:toaster_with_breads") === 3) {
                block.dimension.spawnParticle("ff:toast_smoke", { x: block.location.x + 0.55, y: block.location.y + 0.5, z: block.location.z + 0.50 });
            }
            if (block.permutation.getState("ff:toaster_with_breads") === 4) {
                block.dimension.spawnParticle("ff:toast_smoke", { x: block.location.x + 0.55, y: block.location.y + 0.5, z: block.location.z + 0.50 });

            }


        },
    });
});


world.beforeEvents.worldInitialize.subscribe(ffh => {
    ffh.blockComponentRegistry.registerCustomComponent('ff:water_particle', {
        onTick: e => {
            const { block } = e;
            const { x, y, z } = block.location;
            if (Math.floor(Math.random() * 20) && block.permutation.getState("ff:sink_vars") === 1) {
                block.dimension.spawnParticle("minecraft:water_drip_particle", { x: block.location.x + 0.5, y: block.location.y + 1.2, z: block.location.z + 0.5 });
            }

        },
    });
});

const wooden_support_verticalTag = 'ff:wooden_support_vertical';
class wooden_support_vertical_Manager {
    static updatewooden_support_verticalsAround(block) {
        let aboveBlock = undefined;
        try {
            aboveBlock = block.above(1);
        } catch { }
        let belowBlock = undefined;
        try {
            belowBlock = block.below(1);
        } catch { }
        const blocks = [
            aboveBlock,
            belowBlock,
            block
        ];
        for (const wooden_support_vertical of blocks) {
            if (wooden_support_vertical != undefined && wooden_support_vertical.hasTag(wooden_support_verticalTag)) this.updatewooden_support_vertical(wooden_support_vertical);
        }
    }
    static updatewooden_support_vertical(block) {
        let aboveBlock = undefined;
        try {
            aboveBlock = block.above(1);
        } catch { }
        let belowBlock = undefined;
        try {
            belowBlock = block.below(1);
        } catch { }
        if (aboveBlock != undefined) {
            if (aboveBlock.hasTag(wooden_support_verticalTag)) {
                block.setPermutation(block.permutation.withState("ff:top_bit", true));
            } else block.setPermutation(block.permutation.withState("ff:top_bit", false));
        } else block.setPermutation(block.permutation.withState("ff:top_bit", false));
        if (belowBlock != undefined) {
            if (belowBlock.hasTag(wooden_support_verticalTag)) {
                block.setPermutation(block.permutation.withState("ff:bottom_bit", true));
            } else block.setPermutation(block.permutation.withState("ff:bottom_bit", false));
        } else block.setPermutation(block.permutation.withState("ff:bottom_bit", false));
    }
}
world.afterEvents.playerBreakBlock.subscribe((data) => {
    wooden_support_vertical_Manager.updatewooden_support_verticalsAround(data.block);
});
world.afterEvents.playerPlaceBlock.subscribe((data) => {
    wooden_support_vertical_Manager.updatewooden_support_verticalsAround(data.block);
});


world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('ff:slab_cinder_on_player_destroy', {
        onPlayerDestroy(e) {
            const { block, player } = e;
            if (!player || !player.getComponent('equippable')) {
                return;
            }
            const selectedItem = player.getComponent('equippable').getEquipment('Mainhand');
            const isPickaxe = selectedItem && selectedItem.hasTag('minecraft:is_axe');
            if (isPickaxe) {
                const slabItem = new ItemStack('ff:cinder_slab', 1);
                e.dimension.spawnItem(slabItem, block.location);
            }
        }
    });
});


world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('ff:slab_cinder_on_interact', {
        onPlayerInteract(e) {
            const { block, player, face } = e;
            console.warn(`Interacted face: ${face}`);
            const equipment = player.getComponent('equippable');
            const selectedItem = equipment.getEquipment('Mainhand');
            if (selectedItem?.typeId === 'ff:cinder_slab' && !block.permutation.getState('ff:double')) {
                const verticalHalf = block.permutation.getState('minecraft:vertical_half');
                const isBottomUp = verticalHalf === 'bottom' && face === 'Up';
                const isTopDown = verticalHalf === 'top' && face === 'Down';
                if (isBottomUp || isTopDown) {
                    if (player.getGameMode() !== "creative") {
                        selectedItem.amount -= 1;
                        if (selectedItem.amount === 0) {
                            equipment.setEquipment('Mainhand', undefined);
                        } else {
                            equipment.setEquipment('Mainhand', selectedItem);
                        }
                    }
                    block.setPermutation(block.permutation.withState('ff:double', true));
                    block.setWaterlogged(false);
                    player.playSound('use.wood');
                }
            }
        }
    });
});

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('ff:slab_spicewood_on_player_destroy', {
        onPlayerDestroy(e) {
            const { block, player } = e;
            if (!player || !player.getComponent('equippable')) {
                return;
            }
            const selectedItem = player.getComponent('equippable').getEquipment('Mainhand');
            const isPickaxe = selectedItem && selectedItem.hasTag('minecraft:is_axe');
            if (isPickaxe) {
                const slabItem = new ItemStack('ff:spicewood_slab', 1);
                e.dimension.spawnItem(slabItem, block.location);
            }
        }
    });
});


world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('ff:slab_spicewood_on_interact', {
        onPlayerInteract(e) {
            const { block, player, face } = e;
            console.warn(`Interacted face: ${face}`);
            const equipment = player.getComponent('equippable');
            const selectedItem = equipment.getEquipment('Mainhand');
            if (selectedItem?.typeId === 'ff:spicewood_slab' && !block.permutation.getState('ff:double')) {
                const verticalHalf = block.permutation.getState('minecraft:vertical_half');
                const isBottomUp = verticalHalf === 'bottom' && face === 'Up';
                const isTopDown = verticalHalf === 'top' && face === 'Down';
                if (isBottomUp || isTopDown) {
                    if (player.getGameMode() !== "creative") {
                        selectedItem.amount -= 1;
                        if (selectedItem.amount === 0) {
                            equipment.setEquipment('Mainhand', undefined);
                        } else {
                            equipment.setEquipment('Mainhand', selectedItem);
                        }
                    }
                    block.setPermutation(block.permutation.withState('ff:double', true));
                    block.setWaterlogged(false);
                    player.playSound('use.wood');
                }
            }
        }
    });
});


world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('ff:cinder_trapdoor_on_interact', {
        onPlayerInteract(e) {
            const { block, player } = e;
            const currentState = block.permutation.getState('ff:open');
            const newOpenState = !currentState;
            const newPermutation = BlockPermutation.resolve(block.typeId, {
                ...block.permutation.getAllStates(),
                'ff:open': newOpenState
            });
            block.setPermutation(newPermutation);
            const sound = currentState ? 'open.wooden_trapdoor' : 'close.wooden_trapdoor';
            player.playSound(sound);
        }
    });
});

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('ff:spicewood_trapdoor_on_interact', {
        onPlayerInteract(e) {
            const { block, player } = e;
            const currentState = block.permutation.getState('ff:open');
            const newOpenState = !currentState;
            const newPermutation = BlockPermutation.resolve(block.typeId, {
                ...block.permutation.getAllStates(),
                'ff:open': newOpenState
            });
            block.setPermutation(newPermutation);
            const sound = currentState ? 'open.wooden_trapdoor' : 'close.wooden_trapdoor';
            player.playSound(sound);
        }
    });
});

function getPreciseRotation(playerYRotation) {
    if (playerYRotation < 0) playerYRotation += 360;
    const rotation = Math.round(playerYRotation / 22.5);
  
    return rotation !== 16 ? rotation : 0;
  }

world.beforeEvents.worldInitialize.subscribe(({
    blockComponentRegistry
  }) => {
    blockComponentRegistry.registerCustomComponent("ff:adv_rot", {
      beforeOnPlayerPlace(event) {
        const {
          player
        } = event;
        if (!player) return;
  
        const blockFace = event.permutationToPlace.getState("minecraft:block_face");
        if (blockFace !== "up") return;
  
        const playerYRotation = player.getRotation().y;
        const rotation = getPreciseRotation(playerYRotation);
  
        event.permutationToPlace = event.permutationToPlace.withState("ff:block_rotation", rotation);
      }
    });
  });

world.beforeEvents.worldInitialize.subscribe(event => {
    event.blockComponentRegistry.registerCustomComponent("ff:christmas_stocking_vars", {
      onPlace: (onPlaceEvent => {
        onPlaceEvent.block.setPermutation(onPlaceEvent.block.permutation.withState("ff:stocking_vars", Math.round(Math.random() * 3)))
      })
    })
  });

  world.beforeEvents.worldInitialize.subscribe(event => {
    event.blockComponentRegistry.registerCustomComponent("ff:gravestone_vars", {
      onPlace: (onPlaceEvent => {
        onPlaceEvent.block.setPermutation(onPlaceEvent.block.permutation.withState("ff:stocking_vars", Math.round(Math.random() * 3)))
      })
    })
  });

/**
 * @param {number} min
 * @param {number} max
 * @returns {number}
 * */
const randomInt = (min, max) => Math.floor(Math.random() * (max - min + 1)) + min;

world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
    blockComponentRegistry.registerCustomComponent("ff:feldspar_ore_xp_reward", {
        onPlayerDestroy({ block, dimension, player }) {
            const xpAmount = randomInt(0, 3);

            for (let i = 0; i < xpAmount; i++) {
                dimension.spawnEntity("minecraft:xp_orb", block.location);
            }
        },
    });
});

let lastInteraction = { player: null, block: null, time: 0 };

world.beforeEvents.playerInteractWithBlock.subscribe((event) => {
    const { block, player } = event;
    const blockType = block.typeId;
    
    // Check if it's one of our target blocks
    if (!['ff:cinder_log', 'ff:cinder_wood', 'ff:spicewood_wood', 'ff:spicewood_log'].includes(blockType)) return;
    
    lastInteraction = {
        player,
        block: {
            x: block.location.x,
            y: block.location.y,
            z: block.location.z,
            face: event.face,
            blockState: block.permutation.getState("minecraft:block_face"),
            type: blockType
        },
        time: Date.now()
    };
});

system.runInterval(() => {
    if (!lastInteraction.player || !lastInteraction.block || Date.now() - lastInteraction.time > 100) return;

    const player = lastInteraction.player;
    const blockLoc = lastInteraction.block;
    const face = lastInteraction.block.face;
    const blockState = lastInteraction.block.blockState;
    const blockType = lastInteraction.block.type;
    
    const equipment = player.getComponent('equippable');
    const selectedItem = equipment.getEquipment('Mainhand');
    
    if (selectedItem?.hasTag('minecraft:is_axe')) {
        system.run(() => {
            let strippedType;
            switch(blockType) {
                case 'ff:cinder_log':
                    strippedType = 'ff:stripped_cinder_log';
                    break;
                case 'ff:cinder_wood':
                    strippedType = 'ff:stripped_cinder_wood';
                    break;
                case 'ff:spicewood_wood':
                    strippedType = 'ff:stripped_spicewood_wood';
                    break;
                case 'ff:spicewood_log':
                    strippedType = 'ff:stripped_spicewood_log';
                    break;
            }
            
            if (blockState) {
                player.dimension.runCommand(`setblock ${blockLoc.x} ${blockLoc.y} ${blockLoc.z} ${strippedType} ["minecraft:block_face"="${blockState}"]`);
            } else {
                player.dimension.runCommand(`setblock ${blockLoc.x} ${blockLoc.y} ${blockLoc.z} ${strippedType}`);
            }
            player.playSound('step.wood');
        });
    } else if (selectedItem && selectedItem.typeId !== blockType) {
        const allowedSuffixes = [
            '_log', '_wood', '_planks', '_leaves', '_sapling', '_slab', '_stairs', '_fence', '_door', '_trapdoor', '_button', '_pressure_plate', '_sign', '_wall', '_stripped'
        ];
        const isVanillaBlock = selectedItem.typeId.startsWith('minecraft:') && !selectedItem.typeId.includes('item');
        const isAllowed = allowedSuffixes.some(suf => selectedItem.typeId.endsWith(suf)) || isVanillaBlock;
        if (!isAllowed) return;

        let targetX = blockLoc.x, targetY = blockLoc.y, targetZ = blockLoc.z;

        switch (face) {
            case 'up': targetY++; break;
            case 'down': targetY--; break;
            case 'north': targetZ--; break;
            case 'south': targetZ++; break;
            case 'east': targetX++; break;
            case 'west': targetX--; break;
        }

        system.run(() => {
            const targetBlock = player.dimension.getBlock({ x: targetX, y: targetY, z: targetZ });
            if (targetBlock && targetBlock.typeId === 'minecraft:air') {
                player.dimension.runCommand(`setblock ${targetX} ${targetY} ${targetZ} ${selectedItem.typeId.replace('minecraft:', '')}`);
                
                if (player.getGameMode() !== "creative") {
                    selectedItem.amount--;
                    if (selectedItem.amount <= 0) {
                        equipment.setEquipment('Mainhand', undefined);
                    } else {
                        equipment.setEquipment('Mainhand', selectedItem);
                    }
                }
                player.playSound('use.wood');
            }
        });
    }
    
    lastInteraction = { player: null, block: null, time: 0 };
}, 1);

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('ff:check_leaves_survival', {
        onRandomTick(e) {
            const { block } = e;
            const { x, y, z } = block.location;
            
            if (block.permutation.getState("ff:un_natured")) return;
            
            const searchRadius = 6;
            let foundLog = false;
            
            for (let dx = -searchRadius; dx <= searchRadius && !foundLog; dx++) {
                for (let dy = -searchRadius; dy <= searchRadius && !foundLog; dy++) {
                    for (let dz = -searchRadius; dz <= searchRadius && !foundLog; dz++) {
                        const checkBlock = block.dimension.getBlock({
                            x: x + dx,
                            y: y + dy,
                            z: z + dz
                        });
                        
                        if (checkBlock && checkBlock.typeId === 'ff:spicewood_log') {
                            foundLog = true;
                        }
                    }
                }
            }
            
            if (!foundLog && !block.permutation.getState("ff:with_log")) {
                block.dimension.runCommand(`setblock ${x} ${y} ${z} air`);
            }
        },
        beforeOnPlayerPlace(e) {
            if (!e.permutationToPlace.getState("ff:un_natured")) {
                e.permutationToPlace = e.permutationToPlace.withState("ff:un_natured", true);
            }
        },
        onPlayerDestroy(e) {
            const { block } = e;
            block.dimension.spawnParticle('sd:spicewood_leaves_particles_step', {
                x: block.location.x + 0.5,
                y: block.location.y + 0.5,
                z: block.location.z + 0.5
            });
        },
        onDestroy(e) {
            const { block } = e;
            block.dimension.spawnParticle('sd:spicewood_leaves_particles_step', {
                x: block.location.x + 0.5,
                y: block.location.y + 0.5,
                z: block.location.z + 0.5
            });
        }
    });
});
