import * as server from "@minecraft/server";

server.world.beforeEvents.worldInitialize.subscribe(result => {
  result.blockComponentRegistry.registerCustomComponent("ff:rotation", {
    beforeOnPlayerPlace: result => {
      let direction = 0;
      try {
        let rotationAngle = result.player.getRotation().y;
        direction = rotationAngle >= -45 && rotationAngle < 45 ? 3 :
                    rotationAngle >= 45 && rotationAngle < 135 ? 4 :
                    rotationAngle >= -135 && rotationAngle < -45 ? 5 : 2;
      } catch (e) { }
      let state = result.block.permutation.getAllStates();
      state["block:rotation"] = direction;
      result.permutationToPlace = server.BlockPermutation.resolve(result.permutationToPlace.type.id, state);
    }
  });

  result.blockComponentRegistry.registerCustomComponent("ff:pan", {
    onTick: result => {
      let state = result.block.permutation.getAllStates();
      if (result.block.below(1).hasTag("stove_on"))
        state["block:stove_below"] = true;
      else if (result.block.below(1).hasTag("stove_off"))
        state["block:stove_below"] = false;
      
      let permutation = server.BlockPermutation.resolve(result.block.typeId, state);
      result.block.setPermutation(permutation);

      if (Math.random() < 0.09 && state["block:stove_below"] === true) {
        result.block.dimension.runCommand(
          `particle ff:pan_smoke ${result.block.location.x} ${result.block.location.y + 0.1} ${result.block.location.z}`
        );
      }
      if (Math.random() < 0.2 && state["block:stove_below"] === true) {
        result.block.dimension.runCommand(
          `particle ff:pan_oil ${result.block.location.x} ${result.block.location.y + 0.1} ${result.block.location.z}`
        );
      }
      result.block.setPermutation(server.BlockPermutation.resolve(result.block.typeId, state));
    },

    onPlayerInteract: result => {
      if (result.player.isSneaking || result.face !== "Up") return;
      let selectedQuadrant;
      let x = ((result.faceLocation.x % 1) + 1) % 1;
      let z = ((result.faceLocation.z % 1) + 1) % 1;

      if (x > 0 && x < 0.5 && z > 0.5 && z < 1) {
        selectedQuadrant = "bottom_left";
      } else if (x > 0.5 && x < 1 && z > 0.5 && z < 1) {
        selectedQuadrant = "bottom_right";
      } else if (x > 0 && x < 0.5 && z > 0 && z < 0.5) {
        selectedQuadrant = "top_left";
      } else if (x > 0.5 && x < 1 && z > 0 && z < 0.5) {
        selectedQuadrant = "top_right";
      }

      let item = result.player.getComponent("minecraft:equippable").getEquipment("Mainhand");
      let entities = result.block.dimension.getEntitiesAtBlockLocation(result.block.center())
                      .filter(entity => entity.typeId !== "minecraft:player");
      let interactEntity = null;
      const typeQuadrantMap = {
        "ff:pan_bottom_left": "bottom_left",
        "ff:pan_bottom_right": "bottom_right",
        "ff:pan_top_left": "top_left",
        "ff:pan_top_right": "top_right"
      };

      for (let entity of entities) {
        if (typeQuadrantMap[entity.typeId] === selectedQuadrant) {
          interactEntity = entity;
          break;
        }
      }

      if (!interactEntity && item) {
        const center = result.block.center();
        const offsetMapping = {
          "top_left": { x: -0.15, z: -0.15 },
          "top_right": { x: 0.15, z: -0.15 },
          "bottom_left": { x: -0.15, z: 0.15 },
          "bottom_right": { x: 0.15, z: 0.15 }
        };
        const off = offsetMapping[selectedQuadrant] || { x: 0, z: 0 };
        const spawnPos = {
          x: center.x + off.x,
          y: center.y + -0.5,
          z: center.z + off.z,
        };
        interactEntity = result.block.dimension.spawnEntity("ff:pan_" + selectedQuadrant, spawnPos);
      }

      if (!result.player.getComponent("minecraft:equippable").getEquipment("Mainhand")) {
        if (interactEntity && interactEntity.getTags().length > 0) {
          interactEntity.getTags().forEach(tag => {
            result.player.getComponent(server.EntityInventoryComponent.componentId)
                  .container.addItem(new server.ItemStack(tag, 1));
            interactEntity.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 air`);
            interactEntity.removeTag(tag);
          });
        }
        return;
      } else { 
        let stop = false;
        if (interactEntity) {
          interactEntity.getTags().forEach(tag => {
            if (item.typeId === tag) {
              result.player.getComponent(server.EntityInventoryComponent.componentId)
                    .container.addItem(new server.ItemStack(item.typeId, 1));
              interactEntity.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 air`);
              stop = true;
            } else {
              result.player.getComponent(server.EntityInventoryComponent.componentId)
                    .container.addItem(new server.ItemStack(tag, 1));
            }
            interactEntity.removeTag(tag);
          });
        }
        if (stop) return;

        if (interactEntity) {
          interactEntity.setRotation({
            x: result.player.getRotation().x,
            y: result.player.getRotation().y
          });
          interactEntity.addTag(item.typeId);
          interactEntity.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 ${item.typeId} 1`);
        }

        if (item.amount === 1) {
          result.player.getComponent(server.EntityInventoryComponent.componentId)
                .container.setItem(result.player.selectedSlotIndex, undefined);
        } else {
          item.amount--;
          result.player.getComponent(server.EntityInventoryComponent.componentId)
                .container.setItem(result.player.selectedSlotIndex, item);
        }

        if (result.block.below(1).hasTag("stove_on") && interactEntity) {
            interactEntity.runCommand(`event entity @s ff:cook_timer`);
            interactEntity.dimension.playSound("add_food", interactEntity.location);
            interactEntity.dimension.playSound("mob.zombie.remedy", interactEntity.location, { volume: 0.5 });
            
            const cookingEntity = interactEntity;
            server.system.runTimeout(() => {
                try {
                  const tags = cookingEntity.getTags();
                  const inputItem = tags.length > 0 ? tags[0] : null;
                  const conversion = {
                    "minecraft:kelp": "minecraft:dried_kelp",
                    "minecraft:potato": "minecraft:baked_potato",
                    "minecraft:beef": "minecraft:cooked_beef",
                    "minecraft:rotten_flesh": "minecraft:leather",
                    "minecraft:porkchop": "minecraft:cooked_porkchop",
                    "minecraft:rabbit": "minecraft:cooked_rabbit",
                    "minecraft:cod": "minecraft:cooked_cod",
                    "minecraft:salmon": "minecraft:cooked_salmon",
                    "minecraft:mutton": "minecraft:cooked_mutton",
                    "minecraft:melon_slice": "minecraft:glistering_melon_slice",
                    "minecraft:chicken": "minecraft:cooked_chicken"
                  };
                  
                  if (inputItem) {
                    const outputItem = conversion[inputItem] !== undefined ? conversion[inputItem] : inputItem;
                    const pos = cookingEntity.location;
                    const spawnPos = {
                      x: pos.x + 0.5,
                      y: pos.y + 0.5,
                      z: pos.z + 0.5
                    };
                    
                    cookingEntity.dimension.spawnItem(new server.ItemStack(outputItem, 1), spawnPos);
                  }
                  
                  if (inputItem) {
                    cookingEntity.removeTag(inputItem);
                  }
                  
                  cookingEntity.triggerEvent("ff:kill");
                } catch (e) {
                  console.error("ERR ERROR", e);
                }
                
                server.system.runTimeout(() => {
                  result.player.runCommand(
                    `execute positioned ${result.block.center().x} ${result.block.center().y} ${result.block.center().z} run tp @e[type=item,r=1.1] ${result.block.center().x} ${result.block.center().y} ${result.block.center().z}`
                  );
                }, 1);
              }, 600);       
        }
      }
    }
  });

  result.blockComponentRegistry.registerCustomComponent("ff:stove", {
    onPlayerInteract: result => {
      let state = result.block.permutation.getAllStates();
      let item = result.player.getComponent("minecraft:equippable").getEquipment("Mainhand");
      if (state["block:lit"] === true && item?.typeId === "minecraft:water_bucket") {
        state["block:lit"] = false;
        result.player.getComponent("minecraft:equippable")
              .setEquipment("Mainhand", new server.ItemStack("minecraft:bucket", 1));
        result.player.dimension.playSound("extinguish.candle", result.block.location);
      } else if (state["block:lit"] === false && item?.typeId === "minecraft:flint_and_steel") {
        state["block:lit"] = true;
        if (item.getComponent("minecraft:durability").damage < item.getComponent("minecraft:durability").maxDurability) {
          item.getComponent("minecraft:durability").damage++;
          result.player.getComponent("minecraft:equippable").setEquipment("Mainhand", item);
        } else {
          result.player.getComponent("minecraft:equippable").setEquipment("Mainhand", undefined);
          result.player.dimension.playSound("random.break", result.player.location);
        }
        result.player.dimension.playSound("fire.ignite", result.block.location);
      }
      result.block.setPermutation(server.BlockPermutation.resolve(result.block.typeId, state));
    }
  });
});

server.world.beforeEvents.worldInitialize.subscribe(result => {
  result.blockComponentRegistry.registerCustomComponent("ff:cutting_board", {
    onPlayerInteract: result => {
      const { block, player, face } = result;
      if (player.isSneaking || face !== "Up") return;

      const equippable = player.getComponent("minecraft:equippable");
      let item = equippable.getEquipment("Mainhand");
      const inv = player.getComponent(server.EntityInventoryComponent.componentId).container;
      const dim = block.dimension;
      const center = block.center();

      const mapping = { 
        "minecraft:bread": ["ff:bread_slice", 2, 4],
        "minecraft:melon_block": ["minecraft:melon_slice", 4, 6],
        "minecraft:sugar_cane": ["minecraft:sugar", 2, 4]
       };
      const knifeTypes = [
        "ff:iron_knife",
        "ff:diamond_knife",
        "ff:netherite_knife",
        "ff:gold_knife"
      ];

      let entity = dim.getEntitiesAtBlockLocation(center)
                      .find(e => e.typeId === "ff:entity_cutting_board");

      if (item && mapping[item.typeId] && !entity) {
        entity = dim.spawnEntity("ff:entity_cutting_board", {
          x: center.x, y: center.y - 0.5, z: center.z
        });
        entity.addTag(item.typeId);
        entity.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 ${item.typeId} 1`);
        const slot = player.selectedSlotIndex;
        if (item.amount > 1) {
          item.amount--;
          inv.setItem(slot, item);
        } else {
          inv.setItem(slot, undefined);
        }
        return;
      }

      if (item && knifeTypes.includes(item.typeId) && entity) {
        const tag = entity.getTags()[0];
        const recipe = mapping[tag] || [];
        const [sliceId, min, max] = recipe;
        if (sliceId && Number.isInteger(min) && Number.isInteger(max)) {
          const count = Math.floor(Math.random() * (max - min + 1)) + min;
          for (let i = 0; i < count; i++) {
            inv.addItem(new server.ItemStack(sliceId, 1));
          }
        }
        player.dimension.playSound("mob.sheep.shear", player.location);
        
        const durComp = item.getComponent("minecraft:durability");
        if (durComp) {
          durComp.damage++;
          if (durComp.damage >= durComp.maxDurability) {
            equippable.setEquipment("Mainhand", undefined);
            player.dimension.playSound("random.break", player.location);
          } else {
            equippable.setEquipment("Mainhand", item);
          }
        }

        entity.runCommand(`particle ff:sparks_done ${block.location.x} ${block.location.y} ${block.location.z}`);
        entity.runCommand(`event entity @s kills`);
        dim.runCommand(
          `execute positioned ${block.location.x} ${block.location.y} ${block.location.z} run event entity @e[r=0.3] kill`
        );
      }
    }
  });
});